@@ -7,7 +7,7 @@
 -- index. Please note that tracking index validity is out of scope for this
 -- library: this is responsibility of your application. It is advisable that in
 -- your validity tracking code, you also check for format validity (see
--- "indexFormatValid") and scrap and re-create index when needed.
+-- 'indexFormatValid') and scrap and re-create index when needed.
 --
 -- The index is a binary file that overlays a hashed tree over the working
 -- copy. This means that every working file and directory has an entry in the
@@ -17,7 +17,7 @@
 --
 -- There are two entry types, a file entry and a directory entry. Both have a
 -- common binary format (see 'Item'). The on-disk format is best described by
--- 'peekItem'.
+-- the section /Index format/ below.
 --
 -- For each file, the index has a copy of the file's last modification
 -- timestamp taken at the instant when the hash has been computed. This means
@@ -35,6 +35,24 @@
 -- when a file in a directory changes hash, this triggers recomputation of all
 -- of its parent directory hashes; moreover this is done efficiently -- each
 -- directory is updated at most once during an update run.
+--
+-- /Index format/
+--
+-- The Index is organised into \"lines\" where each line describes a single
+-- indexed item. Cf. 'Item'.
+--
+-- The first word on the index \"line\" is the length of the file path (which is
+-- the only variable-length part of the line). Then comes the path itself, then
+-- fixed-length hash (sha256) of the file in question, then two words, one for
+-- size and one "aux", which is used differently for directories and for files.
+--
+-- With directories, this aux holds the offset of the next sibling line in the
+-- index, so we can efficiently skip reading the whole subtree starting at a
+-- given directory (by just seeking aux bytes forward). The lines are
+-- pre-ordered with respect to directory structure -- the directory comes first
+-- and after it come all its items. Cf. 'readIndex''.
+--
+-- For files, the aux field holds a timestamp.
 
 module Storage.Hashed.Index( readIndex, updateIndexFrom, indexFormatValid
                            , updateIndex , Index, filter )
@@ -50,13 +68,14 @@ import Bundled.Posix( getFileStatusBS, modificationTime,
                       getFileStatus, fileSize, fileExists )
 import System.IO.MMap( mmapFileForeignPtr, mmapFileByteString, Mode(..) )
 import System.IO( )
-import System.Directory( doesFileExist )
+import System.Directory( doesFileExist, getCurrentDirectory )
 #if mingw32_HOST_OS
 import System.Directory( renameFile )
 import System.FilePath( (<.>) )
 #else
 import System.Directory( removeFile )
 #endif
+import System.FilePath( (</>) )
 
 import Control.Monad( when )
 import Control.Exception.Extensible
@@ -146,7 +165,7 @@ xlatePoke64 ptr v = poke ptr (xlate64 v)
 -- the item can be filled out using 'update'.
 createItem :: ItemType -> AnchoredPath -> ForeignPtr () -> Int -> IO Item
 createItem typ apath fp off =
- do let dsc = BS.concat [ BSC.singleton $ (typ == TreeType) ? ('D', 'F')
+ do let dsc = BS.concat [ BSC.singleton $ if typ == TreeType then 'D' else 'F'
                         , flatten apath
                         , BS.singleton 0 ]
         (dsc_fp, dsc_start, dsc_len) = toForeignPtr dsc
@@ -158,22 +177,10 @@ createItem typ apath fp off =
                       (fromIntegral dsc_len)
                peekItem fp off
 
--- | Read the on-disk representation into internal data structure. The Index is
--- organised into "lines" where each line describes a single indexed
--- item. Cf. 'Item'.
+-- | Read the on-disk representation into internal data structure.
 --
--- The first word on the index "line" is the length of the file path (which is
--- the only variable-length part of the line). Then comes the path itself, then
--- fixed-length hash (sha256) of the file in question, then two words, one for
--- size and one "aux", which is used differently for directories and for files.
---
--- With directories, this aux holds the offset of the next sibling line in the
--- index, so we can efficiently skip reading the whole subtree starting at a
--- given directory (by just seeking aux bytes forward). The lines are
--- pre-ordered with respect to directory structure -- the directory comes first
--- and after it come all its items. Cf. 'readIndex''.
---
--- For files, the aux field holds a timestamp.
+-- See the module-level section /Index format/ for details on how the index
+-- is structured.
 peekItem :: ForeignPtr () -> Int -> IO Item
 peekItem fp off =
     withForeignPtr fp $ \p -> do
@@ -202,7 +209,8 @@ iHash' :: Item -> Hash
 iHash' i = SHA256 (iHash i)
 
 -- | Gives a ForeignPtr to mmapped index, which can be used for reading and
--- updates.
+-- updates. The req_size parameter, if non-0, expresses the requested size of
+-- the index file. mmapIndex will grow the index if it is smaller than this.
 mmapIndex :: forall a. FilePath -> Int -> IO (ForeignPtr a, Int)
 mmapIndex indexpath req_size = do
   exist <- doesFileExist indexpath
@@ -214,11 +222,12 @@ mmapIndex indexpath req_size = do
               | otherwise -> 0
   case size of
     0 -> return (castForeignPtr nullForeignPtr, size)
-    _ -> do (x, _) <- mmapFileForeignPtr indexpath
-                                         ReadWrite (Just (0, size + size_magic))
+    _ -> do (x, _, _) <- mmapFileForeignPtr indexpath
+                                            ReadWriteEx (Just (0, size + size_magic))
             return (x, size)
 
 data IndexM m = Index { mmap :: (ForeignPtr ())
+                      , basedir :: FilePath
                       , hashtree :: Tree m -> Hash
                       , predicate :: AnchoredPath -> TreeItem m -> Bool }
               | EmptyIndex
@@ -273,7 +282,7 @@ readDir index state item =
            nullleaf = null inferiors && oldhash == nullsha
            nullsha = SHA256 (BS.replicate 32 0)
            tree' = makeTree [ (n, fromJust $ treeitem s) | (n, s) <- inferiors, isJust $ treeitem s ]
-           treehash = we_changed ? (hashtree index tree', oldhash)
+           treehash = if we_changed then hashtree index tree' else oldhash
            tree = tree' { treeHash = treehash }
 
        when we_changed $ updateItem item 0 treehash
@@ -290,7 +299,7 @@ readFile index state item =
        size <- xlatePeek64 $ iSize item
        let mtime' = modificationTime st
            size' = fromIntegral $ fileSize st
-           readblob = readSegment (BSC.unpack $ iPath item, Nothing)
+           readblob = readSegment (basedir index </> BSC.unpack (iPath item), Nothing)
            exists = fileExists st
            we_changed = mtime /= mtime' || size /= size'
            hash = iHash' item
@@ -300,7 +309,7 @@ readFile index state item =
                updateTime item mtime'
        return $ Result { changed = not exists || we_changed
                        , next = start state + itemNext item
-                       , treeitem = exists ? (Just $ File $ Blob readblob hash, Nothing)
+                       , treeitem = if exists then Just $ File $ Blob readblob hash else Nothing
                        , resitem = item }
 
 updateIndex :: Index -> IO (Tree IO)
@@ -328,8 +337,10 @@ updateIndex index =
 readIndex :: FilePath -> (Tree IO -> Hash) -> IO Index
 readIndex indexpath ht = do
   (mmap_ptr, mmap_size) <- mmapIndex indexpath 0
+  base <- getCurrentDirectory
   return $ if mmap_size == 0 then EmptyIndex
                              else Index { mmap = mmap_ptr
+                                        , basedir = base
                                         , hashtree = ht
                                         , predicate = \_ _ -> True }
 
