@@ -1,4 +1,4 @@
-{-# LANGUAGE ScopedTypeVariables, BangPatterns, TypeSynonymInstances, UndecidableInstances #-}
+{-# LANGUAGE ScopedTypeVariables, BangPatterns, TypeSynonymInstances, UndecidableInstances, FlexibleInstances #-}
 
 -- | An experimental monadic interface to Tree mutation. The main idea is to
 -- simulate IO-ish manipulation of real filesystem (that's the state part of
@@ -15,6 +15,7 @@ module Storage.Hashed.Monad
     , tree, TreeState, TreeMonad, TreeIO, runTreeMonad
     , initialState, replaceItem
     , findM, findFileM, findTreeM
+    , TreeRO, TreeRW
     ) where
 
 import Prelude hiding ( readFile, writeFile )
