@@ -171,8 +171,10 @@ tree = [ testCase "modifyTree" check_modify
        , testCase "modifyTree removal" check_modify_remove
        , testCase "expand" check_expand
        , testCase "expandPath" check_expand_path
+       , testCase "expandPath of sub" check_expand_path_sub
        , testCase "diffTrees" check_diffTrees
        , testCase "diffTrees identical" check_diffTrees_ident
+       , testProperty "expandPath" prop_expandPath
        , testProperty "shapeEq" prop_shape_eq
        , testProperty "expandedShapeEq" prop_expanded_shape_eq
        , testProperty "expand is identity" prop_expand_id
@@ -253,6 +255,16 @@ tree = [ testCase "modifyTree" check_modify
             assertBool "badpath not reachable in t'" $
                        badpath `notElem` (map fst $ list t')
 
+          check_expand_path_sub = do
+            t <- expandPath testTree $ floatPath "substub"
+            t' <- expandPath testTree $ floatPath "substub/stub"
+            t'' <- expandPath testTree $ floatPath "subtree/stub"
+            assertBool "leaf is not a Stub" $
+                isNothing (findTree testTree $ floatPath "substub")
+            assertBool "leaf is not a Stub" $ isJust (findTree t $ floatPath "substub")
+            assertBool "leaf is not a Stub (2)" $ isJust (findTree t' $ floatPath "substub/stub")
+            assertBool "leaf is not a Stub (3)" $ isJust (findTree t'' $ floatPath "subtree/stub")
+
           check_diffTrees =
             flip finally (Prelude.writeFile "foo_dir/foo_a" "a\n") $
                  do Prelude.writeFile "foo_dir/foo_a" "b\n"
@@ -307,7 +319,11 @@ tree = [ testCase "modifyTree" check_modify
           prop_overlay_super (t1 :: Tree Identity, t2) =
               (Just LT == runIdentity (t2 `cmpExpandedShape` t1)) ==>
               Just EQ == (runIdentity $ restrict t2 (t1 `overlay` t2) `cmpTree` t2)
-
+          prop_expandPath (TreeWithPath t p) =
+              notStub $ find (runIdentity $ expandPath t p) p
+            where notStub (Just (Stub _ _)) = False
+                  notStub Nothing = error "Did not exist."
+                  notStub _ = True
 
 packed = [ testCase "loose pristine tree" check_loose
          , testCase "load" check_load
@@ -416,12 +432,18 @@ hash = [ testProperty "decodeBase16 . encodeBase16 == id" prop_base16
     where prop_base16 x = (decodeBase16 . encodeBase16) x == x
           prop_base64u x = (decodeBase64u . encodeBase64u) x == x
 
-monad = [ testCase "path expansion" check_virtual ]
+monad = [ testCase "path expansion" check_virtual
+        , testCase "rename" check_rename ]
     where check_virtual = virtualTreeMonad run testTree >> return ()
               where run = do file <- readFile (floatPath "substub/substub/file")
                              file2 <- readFile (floatPath "substub/substub/file2")
                              lift $ BL.unpack file @?= ""
                              lift $ BL.unpack file2 @?= "foo"
+          check_rename = do (_, t) <- virtualTreeMonad run testTree
+                            t' <- darcsAddMissingHashes =<< expand t
+                            forM_ [ (p, i) | (p, i) <- list t' ] $ \(p,i) ->
+                               assertBool ("have hash: " ++ show p) $ itemHash i /= NoHash
+              where run = do rename (floatPath "substub/substub/file") (floatPath "substub/file2")
 
 posix = [ testCase "getFileStatus" $ check_stat Posix.getFileStatus
         , testCase "getSymbolicLinkStatus" $ check_stat Posix.getSymbolicLinkStatus ]
@@ -449,14 +471,6 @@ instance (Arbitrary k, Arbitrary v, Ord k) => Arbitrary (M.Map k v)
 instance Arbitrary BL.ByteString where
     arbitrary = BL.pack `fmap` arbitrary
 
-instance Arbitrary Word32 where
-    arbitrary = do x <- arbitrary :: Gen Int
-                   return $ fromIntegral x
-
-instance Arbitrary Word8 where
-    arbitrary = do x <- arbitrary :: Gen Int
-                   return $ fromIntegral x
-
 instance Arbitrary Hash where
     arbitrary = sized hash'
         where hash' 0 = return NoHash
@@ -466,16 +480,6 @@ instance Arbitrary Hash where
                   0 -> SHA256 . BS.pack <$> sequence [ arbitrary | _ <- [1..32] ]
                   1 -> SHA1 . BS.pack <$> sequence [ arbitrary | _ <- [1..20] ]
 
-instance Arbitrary Word64 where
-    arbitrary = do x <- arbitrary :: Gen Int
-                   y <- arbitrary :: Gen Int
-                   let x' = fromIntegral x
-                       y' = fromIntegral y
-                   return $ x' .|. (y' `shift` 32)
-
-instance Arbitrary Int64 where
-    arbitrary = fromIntegral `fmap` (arbitrary :: Gen Word64)
-
 instance (Monad m) => Arbitrary (TreeItem m) where
   arbitrary = sized tree'
     where tree' 0 = oneof [ return (File emptyBlob), return (SubTree emptyTree) ]
@@ -498,6 +502,14 @@ instance (Monad m) => Arbitrary (Tree m) where
                    Stub _ _ -> arbitrary
                    SubTree t -> return t
 
+data TreeWithPath = TreeWithPath (Tree Identity) AnchoredPath deriving (Show)
+
+instance Arbitrary TreeWithPath where
+  arbitrary = do t <- arbitrary
+                 p <- oneof $ return (AnchoredPath []) :
+                             (map (return . fst) $ list (runIdentity $ expand t))
+                 return $ TreeWithPath t p
+
 ---------------------------
 -- Other instances
 --
