@@ -28,7 +28,8 @@ module Storage.Hashed.Tree
     , modifyTree, updateTree, partiallyUpdateTree, updateSubtrees, overlay
     , addMissingHashes ) where
 
-import Prelude hiding( lookup, filter, all )
+import Control.Exception( catch, IOException )
+import Prelude hiding( lookup, filter, all, catch )
 import Storage.Hashed.AnchoredPath
 import Storage.Hashed.Hash
 
@@ -195,7 +196,7 @@ checkExpand hashFunc t = go (AnchoredPath []) t
         let
             subtree (name, sub) =
                 do let here = path `appendPath` name
-                   sub' <- (Just <$> unstub sub) `catch` \_ -> return Nothing
+                   sub' <- (Just <$> unstub sub) `catch` \(_ :: IOException) -> return Nothing
                    case sub' of
                      Nothing -> return $ Left [(here, treeHash t_, Nothing)]
                      Just sub -> do
@@ -204,10 +205,10 @@ checkExpand hashFunc t = go (AnchoredPath []) t
                               Left problems -> Left problems
                               Right tree -> Right (name, SubTree tree)
             badBlob (_, f@(File (Blob s h))) =
-              fmap (/= h) (hashFunc f `catch` (\_ -> return NoHash))
+              fmap (/= h) (hashFunc f `catch` (\(_ :: IOException) -> return NoHash))
             badBlob _ = return False
             render (name, f@(File (Blob _ h))) = do
-              h' <- (Just <$> hashFunc f) `catch` \_ -> return Nothing
+              h' <- (Just <$> hashFunc f) `catch` \(_ :: IOException) -> return Nothing
               return (path `appendPath` name, h, h')
         subs <- mapM subtree [ x | x@(_, item) <- listImmediate t_, isSub item ]
         badBlobs <- filterM badBlob (listImmediate t) >>= mapM render
